from ..access import acc_task, acc_project
from django import template
from ..models import Task, Project, Comment
from ..views import TaskLike, ProjectLike, CommentLike

register = template.Library()


class xlike_acc(object):
    add_comments = False
    modify = False
    modify_comment = False
    modify_limited = False
    allowed_targetstates = []
    allowed_targetpriority = []


def get_acc(obj, user):
    if type(obj) is Comment:
        return acc_task(obj.task, user)
    elif type(obj) is Task:
        return acc_task(obj, user)
    elif type(obj) is Project:
        return acc_project(obj, user)
    elif type(obj) in [TaskLike, ProjectLike, CommentLike]:
        return xlike_acc()
    else:
        raise AttributeError("Unknown obj of type", type(obj))


@register.simple_tag(name='may_add_comments', takes_context=True)
def may_add_comments(context, obj):
    acc = get_acc(obj, context['request'].user)
    return acc.add_comments


@register.simple_tag(name='may_modify', takes_context=True)
def may_modify(context, obj):
    acc = get_acc(obj, context['request'].user)
    return acc.modify


@register.simple_tag(name='may_modify_comment', takes_context=True)
def may_modify_comment(context, obj):
    acc = get_acc(obj, context['request'].user)
    return acc.modify_comment


@register.simple_tag(name='may_modify_limited', takes_context=True)
def may_modify_limited(context, obj):
    acc = get_acc(obj, context['request'].user)
    return acc.modify_limited


@register.simple_tag(name='targetstates', takes_context=True)
def targetstates(context, obj):
    acc = get_acc(obj, context['request'].user)
    return [str(i) for i in acc.allowed_targetstates]


@register.simple_tag(name='targetpriority', takes_context=True)
def targetpriority(context, obj):
    acc = get_acc(obj, context['request'].user)
    return [str(i) for i in acc.allowed_targetpriority]
